#include "conversionSpaceColor.h"

// convertit de l'espace RGB  l'espace HSV
// les 8 premiers bits de rgb contiennent la couleur rouge
// les 8 suivants la couleur verte
// et les suivants la couleur bleue.
// 0 <= h < 360 o h = -1
// 0 <= s < 1
// 0 <= v < 1

void rgb2hsv(double* h, double* s, double* v, unsigned long rgb )
{
 double Aux ;
 double r,g,b ;
 long ri,gi,bi ;

 ri = rgb & MASK_COLOR ;
 gi = (rgb>> 8) & MASK_COLOR ;
 bi = (rgb>> 16) & MASK_COLOR ;
 r = ri * INV_255 ;
 g = gi * INV_255 ;
 b = bi * INV_255 ;
 if ((ri==gi) && (gi==bi))
	{
	 *h = -1.0;
	 *s = 0.0;
	 *v = (double) r ;
	}
	else
	{
	 if (ri != bi)
		{
		 Aux = 60.0 * (g-b)/(r-b) ;
		 if ((Aux >= 0.0) && (Aux < 60.0))
			{
			 if ((ri != 0) && (ri >= bi))
				{
				 *h = Aux ;
				 *s = (r-b)/r ;
				 *v =  r ;
				 return ;
				}
				else
				{
				 *h = 180.0+Aux ;
				 *s = (b-r)/b ;
				 *v =  b ;
				 return ;
				}
			}
		}
	 if (bi != gi)
		{
		 Aux = 60.0*(r-g)/(b-g) ;
		 if ((Aux >= 0.0) && (Aux < 60.0))
			{
			 if ((gi != 0) && (gi >= bi))
				{
				 *h = 60.0 + Aux ;
				 *s = (g-b)/g ;
				 *v =  g;
				 return ;
				}
				else
				{
				 *h = 240.0 + Aux ;
				 *s = (b-g)/b ;
				 *v =  b ;
				 return ;
				}
			}
		}

	 Aux = 60.0*(b-r)/(g-r) ;
	 if ((gi != 0) && (gi >= ri))
		{
		 *h = 120.0 + Aux ;
		 *s = (g-r)/g ;
		 *v =  g ;
		 return ;
		}
		else
		{
		 *h = 300.0 + Aux ;
		 *s = (r-g)/r ;
		 *v =  r ;
		}
	} //de la fin
}

// Il est ncessaire que :
// 0 <= h < 360 o h = -1
// h = -1 correspond  une couleur achromatique
// 0 <= s < 1
// 0 <= v < 1
// les 8 premiers bits de retour contiennent la couleur rouge
// les 8 suivants la couleur verte
// et les suivants la couleur bleue.
// les derniers sont mis  zros.
unsigned long hsv2rgb( double h, double s, double v )
{
 double aux ;
 long a,b=0 ;

 if (h == -1.0)
	{
	 aux = 255.0 * v + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v ;
	 b = a ;
	 a <<= 8 ;
	 b |= a ;
	 a <<= 8 ;
	 b |= a ;
	 return b ;
	}
 if (h<60.0)
	{
	 aux = 255.0 * v + 0.5 ;
	 b = (long) aux ;
	 //b = (long) 255.0 * v ;
	 aux = 255.0 * v*(1.0+s*(h * DIVIDE_BY_60 - 1.0 )) + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v*(1.0+s*(h * DIVIDE_BY_60 - 1.0 )) ;
	 a <<= 8 ;
	 b |= a ;
	 aux = 255.0 * v*(1.0-s) + 0.5 ;
	 a = (long) aux ;
	 // a = (long) 255.0 * v*(1.0-s) ;
	 a <<= 16 ;
	 b |= a ;
	 return b ;
	}
 if (h<120.0)
	{
	 aux = 255.0 * v*(1.0+s*(1.0 - h * DIVIDE_BY_60 )) + 0.5 ;
	 b   = (long) aux ;
	 //b = (long) 255.0 * v*(1.0+s*(1.0 - h * DIVIDE_BY_60 )) ;
	 aux = 255.0 * v + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v ;
	 a <<= 8 ;
	 b |= a ;
	 aux = 255.0 * v * (1.0 - s) + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v * (1.0 - s) ;
	 a <<= 16 ;
	 b |= a ;
	 return b ;
	}
 if (h<180.0)
	{
	 aux = 255.0 * v * (1.0 - s) + 0.5 ;
	 b = (long) aux ;
	 //b = (long) 255.0 * v * (1.0 - s) ;
	 aux = 255.0 * v + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v ;
	 a <<= 8 ;
	 b |= a ;
	 aux = 255.0 * v*(1.0+s * ( h * DIVIDE_BY_60 -3.0 )) + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v*(1.0+s * ( h * DIVIDE_BY_60 -3.0 )) ;
	 a <<= 16 ;
	 b |= a ;
	 return b ;
	}
 if (h<240.0)
	{
	 aux = 255.0 * v * (1.0 - s) + 0.5 ;
	 b = (long) aux ;
	 //b = (long) 255.0 * v * (1.0 - s) ;
	 aux = 255.0 * v * (1.0 + s * ( 3.0 - h * DIVIDE_BY_60 )) + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v * (1.0 + s * ( 3.0 - h * DIVIDE_BY_60 )) ;
	 a <<= 8 ;
	 b |= a ;
	 aux = 255.0 * v + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v ;
	 a <<= 16 ;
	 b |= a ;
	 return b ;
	}
 if (h<300.0)
	{
	 aux = 255.0 * v*(1.0+s * ( h * DIVIDE_BY_60 - 5.0 )) + 0.5 ;
	 b = (long) aux ;
	 //b = (long) 255.0 * v*(1.0+s * ( h * DIVIDE_BY_60 - 5.0 )) ;
	 aux = 255.0 * v * (1.0 - s) + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v * (1.0 - s) ;
	 a <<= 8 ;
	 b |= a ;
	 aux = 255.0 * v + 0.5 ;
	 a = (long) aux ;
	 //a = (long) 255.0 * v ;
	 a <<= 16 ;
	 b |= a ;
	 return b ;
	}

 aux = 255.0 * v + 0.5 ;
 b = (long) aux ;
 //b = (long) 255.0 * v ;
 aux = 255.0 * v * (1.0 - s) + 0.5 ;
 a = (long) aux ;
 //a = (long) 255.0 * v * (1.0 - s) ;
 a <<= 8 ;
 b |= a ;
 aux = 255.0 * v * (1.0 + s * ( 5.0 - h * DIVIDE_BY_60 )) + 0.5 ;
 a = (long) aux ;
 //a = (long) 255.0 * v * (1.0 + s * ( 5.0 - h * DIVIDE_BY_60 )) ;
 a <<= 16 ;
 b |= a ;
 return b ;
}










